(* ===================================================================

   L0 exception and error reporting function

   Syntax:

       /L0-ERROR: Code?
       Inside?
       Where?
       Why?
       ;

       Code   --> string
       Inside --> /within: string
       Where  --> /at: string
       Why    --> /why: string

=================================================================== *)

local

open Parse HolKernel boolLib

val ErrMsgHeader      = ref "L0-ERROR:" ;
val WarningMsgHeader  = ref "L0-WARNING:" ;
val printMsg   = print ;

in

type L0Error = { errorCode   : string option
               , errorWhere  : term option
               , errorInside : term option
               , errorWhy    : string option
               } ;

type L0Warning = { warningCode   : string option
                 , warningWhere  : term option
                 , warningInside : term option
                 , warningWhy    : string option
                 } ;

val dummyL0error = { errorCode   = SOME "dummy"
                   , errorWhere  = NONE
                   , errorInside = NONE
                   , errorWhy    = NONE } ;

exception L0Exn of L0Error ;

fun printL0Msg header (code,waar,inside,why) =

    (printMsg  ("\n" ^ !header) ;

     case code of
           SOME c => printMsg c
           | _    => () ;

     case inside of
           SOME u => (printMsg "\n\n/within:\n\n" ; 
                      printMsg (term_to_string u))
           | _      => () ; 

     case waar of
           SOME u => (printMsg "\n\n/at:\n\n" ; 
                      printMsg (term_to_string u))
           | _      => () ;

     case why of
           SOME r => printMsg ("\n\n/why:\n\n" ^ r )
           | _      => () ;

     printMsg "\n;" ) ;

fun printL0Error (e:L0Error) = printL0Msg ErrMsgHeader (#errorCode   e
                                             ,#errorWhere  e
                                             ,#errorInside e
                                             ,#errorWhy    e ) ;

fun printL0Warning (w:L0Warning) = printL0Msg WarningMsgHeader 
                                  (#warningCode   w
                                  ,#warningWhere  w
                                  ,#warningInside w
                                  ,#warningWhy    w ) ;


end ;
