(* =============================================================

   the signature of a syntax checker 

============================================================= *)
signature SChecker =
   sig

   local

   open Parse HolKernel 
   open Err L0_emb_mldata 

   in

   type SCstate    (* the state to thread reported syntax-errors 
                      and warning *)
   type SCoption   (* the state to keep Checker's options *)
   type SCmsg      (* syntax error/warning information *)

   val option    : SCoption ref 
   (* SC's state *)
   val state      : SCstate ref 
   val initializeState : unit -> unit                  

   (* adding, retrieving, printing reported errors and warnings to/from the state *)
   val add_SCmsgs   : SCmsg list -> unit            
   val get_SCmsgs   : unit -> SCmsg list 
   val print_SCmsg  : SCmsg -> unit
   (* for casting L0Error and L0Warning to SCmsg *)
   val L0Error2SCmsg   : L0Error   -> SCmsg
   val L0Warning2SCmsg : L0Warning -> SCmsg

   (* for checking and collecting errors in fractions of L0
      which are still left unspecified by L0 syntax           *)

   val check_expr          : term -> SCmsg list
   val check_assertion     : term -> SCmsg list   (* does not allow "ret" key-words *)
   val check_assertion_ret : term -> SCmsg list   (* allow "ret" key-word *)
   val check_actual_param  : term -> SCmsg list
   val check_allowedtype   : term -> SCmsg list

   val scAlgebra  : unit alg_INSTR_               

   end 
end 

