(* ===================================================================

   A functor to build a syntax checker for test-suite out of the
   specification for an L0-syntax-checker.

=================================================================== *)

local 

open Parse HolKernel boolLib 
open Susp Err 
open L0_emb_mldata L0_emb_basic
open SChecker Test_emb_mldata Test_emb_basic

in


functor build_TSC (SC : SChecker) =

   struct

   local

   fun println s = (print "\n"; print s)

   (* constructing a modified SC algebra, which is the same as the base
      SC's algebra, except that it is altered for "assert" to 
      generate warning of an asserting is not a simple expression *)

   val scAlgAlt =  { fprint  = #fprint SC.scAlgebra
                   , fasg    = #fasg   SC.scAlgebra
                   , fseq    = #fseq   SC.scAlgebra
                   , fcall   = #fcall  SC.scAlgebra
                   , fif     = #fif    SC.scAlgebra
                   , flocal  = #flocal SC.scAlgebra
                   , fwhile  = #fwhile SC.scAlgebra
                   , fassert = 
                       let
                       fun cassert self a 
                           =
                           ( ( if null (SC.check_expr a) then ()
                               else SC.add_SCmsgs
                                    [ SC.L0Warning2SCmsg
                                    { warningCode   = SOME "build_TSC"
                                    , warningInside = SOME (force self)
                                    , warningWhere  = SOME a
                                    , warningWhy = SOME "a complex assertion whithin a test; it will be skipped during the test" }
                                  ] )
                          ; (#fassert SC.scAlgebra) self a
                          )
                      in
                      cassert end
                  }
                                  

   val SC_instr = fold_INSTR scAlgAlt

   (* the algebra to fold over the structure of TEST *)
   val scTestAlg = { ftest  = let
                              fun sc _ _ i = SC_instr (buildINSTRtree i)
                              in sc end
                   , fsuite = let
                              fun sc _ _ s = (s ; () )
                              in sc end
                   }

   in

   fun print_msgs () 
       =
       ( println ""
       ; (let
          val msgs = List.rev (SC.get_SCmsgs ())
          in
          if null msgs then (println "No syntax errors found" ; println "")
          else ( println "The following syntax errors/warnings are found:"
              ; println ""
              ; List.map SC.print_SCmsg msgs
              ; () )
          end)
       ; println ""
       )

   fun check t = (  SC.initializeState ()
                 ;  ((fold_TEST scTestAlg o buildTEST o remove_whitebox) t
                     handle 
                     L0Exn e => SC.add_SCmsgs [SC.L0Error2SCmsg e] )
                 ;   print_msgs () )

   end (* of inner-local *)
   end (* of the structure *)

end